/* Program:        lj2ps
** File:	   transform.c
**
** Author:	   Christopher Lishka
** Organization:   Wisconsin State Laboratory of Hygiene
**		   Data Processing Dept.
**
** Copyright (C) 1990 by Christopher Lishka.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 1, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

static char * ModuleID = "Module transform: v1.0.1.2, production";

  /* Include files
  */
#include <stdio.h>
#include "trans.h"
#include "scan.h"
#include "lj2ps.h"
#include "lj.h"

  /* External definitions
  */

  /* Global variables
  */

  /* Global function list
  */
extern void  transform();

  /* Local constants
  */

  /* Local structures and types
  */

  /* Local variables
  */

  /* Local macro definitions
  */
#define report(x)  fprintf(output_file, "%s", x);

  /* Local function list
  */

  /* Function bodies
  */

  /* transform() reads text mixed with laserjet commands from its input
  ** and "magically" changes it to postscript on its output.  The magic
  ** is performed in a tight while-loop which acts as a simple decision
  ** table: for each token type (command, text, parameter) the appropriate
  ** action is done (which usually involves writing to stdout or calling
  ** an emulation function).
  */
void
transform(input_file, output_file)
    FILE *input_file, *output_file;
{

    /* Reset the scanner to read from the input */
  scan_init(input_file);

    /* While the input stream still has tokens, handle each token through
    ** the switch() jump table.
    */
  scan_state = SS_TEXT;    		/* Set the initial scan state */
  while( scan() ){
#ifdef DEBUG
    if( debug ) fprintf(stderr, "{TOKEN:%d}", curr_token.code);
#endif
    switch( curr_token.code ){
    case TEXT_CHAR:
      if( (page_height - current_y) > (text_height + margin_top + LJ_ERROR) ){
	lj_page_end(output_file);
	lj_page_begin(output_file);
      }
      lj_text_add(output_file, text);
      break;
    case TEXT_NULL:		/* Ignore nulls */
      warning("encountered a null; ignoring", "");
      break;
    case TEXT_lp:
      lj_text_add(output_file, "\\(");
      break;
    case TEXT_rp:
      lj_text_add(output_file, "\\)");
      break;
    case TEXT_bslash:
      lj_text_add(output_file, "\\\\");
      break;
    case TEXT_NEWLINE:
      if( (page_height - current_y) > (text_height + margin_top + LJ_ERROR) ){
	lj_page_end(output_file);
	lj_page_begin(output_file);
      }
      lj_nl(output_file);	/* Write out a newline */
      current_line++;
      break;
    case TEXT_FORMFEED:
      lj_page_end(output_file);
      /* XXX insert page counting mechanism here! */
      lj_page_begin(output_file);
      break;
    case TEXT_TAB:
        /* XXX This should check for either font_p or font_s, and not assume
	**     font_p automatically!
	*/
      if( font_p.spacing == LJ_SP_PROPORTIONAL )
	warning("a tab is being used with a proportional font!", "");
      lj_text_end(output_file);
      fputs("TAB ", output_file);
      lj_text_begin();
      break;
    case TEXT_SH_IN:
      warning("shift-in is not yet supported -- ignoring", "");
      break;
    case TEXT_SH_OUT:
      warning("shift-out is not yet su